@@ -8,25 +8,28 @@ from matplotlib.collections import PatchCollection
 from matplotlib.lines import Line2D
 
 from scipy.ndimage.interpolation import geometric_transform
-
+import ase
 import numpy as np
+import sidpy
 
-###################################
+# ##################################
 # Plot Reciprocal Unit Cell in 2D #
-###################################
+# ##################################
 
 
-def plot_reciprocal_unit_cell_2D(tags):
-    # Plot # unit cell in reciprocal space in 2D
-    reciprocal_unit_cell = tags['reciprocal_unit_cell']
+def plot_reciprocal_unit_cell_2D(atoms):
+    """Plot # unit cell in reciprocal space in 2D"""
+    
+    
+    reciprocal_unit_cell = atoms.get_reciprocal_cell()
 
     # ignore y direction
 
     x = [reciprocal_unit_cell[0, 0], reciprocal_unit_cell[0, 0], reciprocal_unit_cell[1, 0], reciprocal_unit_cell[1, 0]]
     z = [reciprocal_unit_cell[0, 2], reciprocal_unit_cell[2, 2], reciprocal_unit_cell[2, 2], reciprocal_unit_cell[0, 2]]
 
-    print(x, z)
-    print(reciprocal_unit_cell)
+    #print(x, z)
+    #print(reciprocal_unit_cell)
 
     # Plot 2D
     fig = plt.figure()
@@ -66,78 +69,86 @@ def plot_reciprocal_unit_cell_2D(tags):
     # texfig.savefig("recip_unit_cell")
     # fig.savefig('recip_unit_cell.jpg', dpi=90, bbox_inches='tight')
     plt.show()
+    return fig
 
 
-#####################
+# ####################
 # Plot SAED Pattern #
-#####################
-def plotSAED_parameter(tags, gray=False):
-    tags['convergence_angle_nm-1'] = 0
+# ####################
+def plotSAED_parameter(gray=False):
+
+    tags = {}
+    tags['convergence_angle_A-1'] = 0
 
     tags['background'] = 'white'  # 'white'  'grey'
-    tags['color map'] = 'plasma'  # ,'cubehelix'#'Greys'#'plasma'
-    tags['color reflections'] = 'intensity'
+    tags['color_map'] = 'plasma'  # ,'cubehelix'#'Greys'#'plasma'
+    tags['color_reflections'] = 'intensity'
 
     if gray:
-        tags['color map'] = 'gray'
+        tags['color_map'] = 'gray'
         tags['background'] = '#303030'  # 'darkgray'
-        tags['color reflections'] = 'intensity'
-    tags['plot HOLZ'] = 0
-    tags['plot HOLZ excess'] = 0
-    tags['plot Kikuchi'] = 1
-    tags['plot reflections'] = 1
+        tags['color_reflections'] = 'intensity'
+    tags['plot_HOLZ'] = 0
+    tags['plot_HOLZ_excess'] = 0
+    tags['plot_Kikuchi'] = 1
+    tags['plot_reflections'] = 1
 
-    tags['color Kikuchi'] = 'green'
+    tags['color_Kikuchi'] = 'green'
 
-    tags['linewidth HOLZ'] = -1  # -1: linewidth according to intensity (structure factor F^2
-    tags['linewidth Kikuchi'] = -1  # -1: linewidth according to intensity (structure factor F^2
+    tags['linewidth_HOLZ'] = -1  # -1: linewidth according to intensity (structure factor F^2
+    tags['linewidth_Kikuchi'] = -1  # -1: linewidth according to intensity (structure factor F^2
 
-    tags['label HOLZ'] = 0
-    tags['label Kikuchi'] = 0
-    tags['label reflections'] = 0
+    tags['label_HOLZ'] = 0
+    tags['label_Kikuchi'] = 0
+    tags['label_reflections'] = 0
 
-    tags['label color'] = 'white'
-    tags['label size'] = 10
+    tags['label_color'] = 'white'
+    tags['label_size'] = 10
 
-    tags['color Laue Zones'] = ['red', 'blue', 'green', 'blue', 'green']  # , 'green', 'red'] #for OLZ give a sequence
-    tags['color zero'] = 'red'  # 'None' #'white'
-    tags['color ring zero'] = 'None'  # 'Red' #'white' #, 'None'
-    tags['width ring zero'] = 2
+    tags['color_Laue_Zones'] = ['red', 'blue', 'green', 'blue', 'green']  # , 'green', 'red'] #for OLZ give a sequence
+    tags['color_zero'] = 'red'  # 'None' #'white'
+    tags['color_ring_zero'] = 'None'  # 'Red' #'white' #, 'None'
+    tags['width_ring_zero'] = .2
 
     # plotDiffPattern(tags,True)
-    tags['plot rotation'] = 0.  # degree
-    tags['plot shift x'] = -0.0
-    tags['plot shift y'] = 0.0
+    tags['plot_rotation'] = 0.  # degree
+    tags['plot_shift_x'] = -0.0
+    tags['plot_shift_y'] = 0.0
+    
+    return tags
 
 
 ########################
 # Plot Kikuchi Pattern #
 ########################
 def plotKikuchi(tags, grey=False):
+    tags = {}
     tags['background'] = 'black'  # 'white'  'grey'
-    tags['color map'] = 'plasma'  # ,'cubehelix'#'Greys'#'plasma'
-    tags['color reflections'] = 'intensity'
+    tags['color_map'] = 'plasma'  # ,'cubehelix'#'Greys'#'plasma'
+    tags['color_reflections'] = 'intensity'
 
-    tags['plot HOLZ'] = 0
-    tags['plot HOLZ excess'] = 0
-    tags['plot Kikuchi'] = 1
-    tags['plot reflections'] = 1
+    tags['plot_HOLZ'] = 0
+    tags['plot_HOLZ_excess'] = 0
+    tags['plot_Kikuchi'] = 1
+    tags['plot_reflections'] = 1
 
-    tags['label HOLZ'] = 0
-    tags['label Kikuchi'] = 0
-    tags['label reflections'] = 0
+    tags['label_HOLZ'] = 0
+    tags['label_Kikuchi'] = 0
+    tags['label_reflections'] = 0
 
-    tags['label color'] = 'white'
-    tags['label size'] = 10
+    tags['label_color'] = 'white'
+    tags['label_size'] = 10
 
-    tags['color Kikuchi'] = 'green'
-    tags['linewidth HOLZ'] = -1  # -1: linewidth according to intensity (structure factor F^2
-    tags['linewidth Kikuchi'] = -1  # -1: linewidth according to intensity (structure factor F^2
+    tags['color_Kikuchi'] = 'green'
+    tags['linewidth_HOLZ'] = -1  # -1: linewidth according to intensity (structure factor F^2
+    tags['linewidth_Kikuchi'] = -1  # -1: linewidth according to intensity (structure factor F^2
 
-    tags['color Laue Zones'] = ['red', 'blue', 'green', 'blue', 'green']  # , 'green', 'red'] #for OLZ give a sequence
-    tags['color zero'] = 'white'  # 'None' #'white'
-    tags['color ring zero'] = 'None'  # 'Red' #'white' #, 'None'
-    tags['width ring zero'] = 2
+    tags['color_Laue_Zones'] = ['red', 'blue', 'green', 'blue', 'green']  # , 'green', 'red'] #for OLZ give a sequence
+    tags['color_zero'] = 'white'  # 'None' #'white'
+    tags['color_ring_zero'] = 'None'  # 'Red' #'white' #, 'None'
+    tags['width_ring_zero'] = 2
+    
+    return tags
 
     # plotDiffPattern(tags,True)
 
@@ -146,73 +157,76 @@ def plotKikuchi(tags, grey=False):
 # Plot HOLZ Pattern #
 ########################
 
-def plotHOLZ_parameter(tags, grey=False):
+def plotHOLZ_parameter(grey=False):
+    tags = {}
     tags['background'] = 'gray'  # 'white'  'grey'
-    tags['color map'] = 'plasma'  # ,'cubehelix'#'Greys'#'plasma'
-    tags['color reflections'] = 'intensity'
+    tags['color_map'] = 'plasma'  # ,'cubehelix'#'Greys'#'plasma'
+    tags['color_reflections'] = 'intensity'
 
-    tags['plot HOLZ'] = 1
-    tags['plot HOLZ excess'] = 1
-    tags['plot Kikuchi'] = 1
-    tags['plot reflections'] = 1
+    tags['plot_HOLZ'] = 1
+    tags['plot_HOLZ_excess'] = 1
+    tags['plot_Kikuchi'] = 1
+    tags['plot_reflections'] = 1
 
-    tags['label HOLZ'] = 0
-    tags['label Kikuchi'] = 0
-    tags['label reflections'] = 0
+    tags['label_HOLZ'] = 0
+    tags['label_Kikuchi'] = 0
+    tags['label_reflections'] = 0
 
-    tags['label color'] = 'white'
-    tags['label size'] = 12
+    tags['label_color'] = 'white'
+    tags['label_size'] = 12
 
-    tags['color Kikuchi'] = 'green'
-    tags['linewidth HOLZ'] = 1  # -1: linewidth according to intensity (structure factor F^2
-    tags['linewidth Kikuchi'] = -1  # -1: linewidth according to intensity (structure factor F^2
+    tags['color_Kikuchi'] = 'green'
+    tags['linewidth_HOLZ'] = 1  # -1: linewidth according to intensity (structure factor F^2
+    tags['linewidth_Kikuchi'] = -1  # -1: linewidth according to intensity (structure factor F^2
 
-    tags['color Laue Zones'] = ['red', 'blue', 'lightblue', 'green', 'red']  # for OLZ give a sequence
-    tags['color zero'] = 'None'  # 'white'
-    tags['color ring zero'] = 'Red'  # 'Red' #'white' #, 'None'
-    tags['width ring zero'] = 2
+    tags['color_Laue_Zones'] = ['red', 'blue', 'lightblue', 'green', 'red']  # for OLZ give a sequence
+    tags['color_zero'] = 'None'  # 'white'
+    tags['color_ring_zero'] = 'Red'  # 'Red' #'white' #, 'None'
+    tags['width_ring_zero'] = 2
 
-    tags['plot rotation'] = 0.  # degree
-    tags['plot shift x'] = -0.0
-    tags['plot shift y'] = 0.0
+    tags['plot_rotation'] = 0.  # degree
+    tags['plot_shift_x'] = -0.0
+    tags['plot_shift_y'] = 0.0
 
     # plotDiffPattern(holz,True)
+    return tags
 
 
 ########################
 # Plot CBED Pattern #
 ########################
 
-def plotCBED_parameter(tags):
+def plotCBED_parameter():
+    tags = {}
     tags['background'] = 'black'  # 'white'  'grey'
-    tags['color map'] = 'plasma'  # ,'cubehelix'#'Greys'#'plasma'
-    tags['color reflections'] = 'intensity'
+    tags['color_map'] = 'plasma'  # ,'cubehelix'#'Greys'#'plasma'
+    tags['color_reflections'] = 'intensity'
 
-    tags['plot HOLZ'] = 1
-    tags['plot HOLZ excess'] = 1
-    tags['plot Kikuchi'] = 1
-    tags['plot reflections'] = 1
+    tags['plot_HOLZ'] = 1
+    tags['plot_HOLZ_excess'] = 1
+    tags['plot_Kikuchi'] = 1
+    tags['plot_reflections'] = 1
 
-    tags['label HOLZ'] = 0
-    tags['label Kikuchi'] = 0
-    tags['label reflections'] = 0
+    tags['label_HOLZ'] = 0
+    tags['label_Kikuchi'] = 0
+    tags['label_reflections'] = 0
 
-    tags['label color'] = 'white'
-    tags['label size'] = 10
+    tags['label_color'] = 'white'
+    tags['label_size'] = 10
 
-    tags['color Kikuchi'] = 'green'
-    tags['linewidth HOLZ'] = -1  # -1: linewidth according to intensity (structure factor F^2
-    tags['linewidth Kikuchi'] = -1  # -1: linewidth according to intensity (structure factor F^2
+    tags['color_Kikuchi'] = 'green'
+    tags['linewidth_HOLZ'] = -1  # -1: linewidth according to intensity (structure factor F^2
+    tags['linewidth_Kikuchi'] = -1  # -1: linewidth according to intensity (structure factor F^2
 
-    tags['color reflections'] = 'intensity'
+    tags['color_reflections'] = 'intensity'
 
-    tags['color Laue Zones'] = ['red', 'blue', 'green']  # , 'green', 'red'] #for OLZ give a sequence
-    tags['color zero'] = 'white'  # 'None' #'white'
-    tags['color ring zero'] = 'Red'  # 'Red' #'white' #, 'None'
-    tags['width ring zero'] = 2
+    tags['color_Laue_Zones'] = ['red', 'blue', 'green']  # , 'green', 'red'] #for OLZ give a sequence
+    tags['color_zero'] = 'white'  # 'None' #'white'
+    tags['color_ring_zero'] = 'Red'  # 'Red' #'white' #, 'None'
+    tags['width_ring_zero'] = 2
 
     # plotDiffPattern(tags,True)
-
+    return tags
 
 ########################
 # Plot HOLZ Pattern #
@@ -323,26 +337,54 @@ def topolar(img, order=1):
     return polar, (rads, angs)
 
 
-def plotRingPattern(tags, grey=False):
+def plot_ring_pattern(atoms, diffraction_pattern=None, grey=False):
+    """
+    Plot of ring diffraction pattern with matplotlib
+    
+    Parameters
+    ----------
+    info: dictionary or sidpy.Dataset
+        information stored as dictionary either directly or in metadata attribute of sidpy.Dataset
+    grey: bool
+        ploting in greyscale if True
+        
+    Returns
+    -------
+    fig: matplotlib figure
+        refereence to matplolib figure
+    """
+    
+    if isinstance(atoms, dict):
+        tags = atoms
+    elif  isinstance(atoms, ase.Atoms):
+        if 'diffraction' in atoms.info:
+            tags = atoms.info['diffraction']
+            plot_diffraction_pattern = True
+        else:
+            raise TypeError('Diffraction information must be in metadata')
+    else:
+        raise TypeError('Diffraction info must be in sidly Dataset or dictionary form')
+        
     d = tags['Ring_Pattern']['allowed']['g norm']
     label = tags['Ring_Pattern']['allowed']['label']
-    if 'label color' not in tags:
-        tags['label color'] = 'navy'
+    if 'label_color' not in tags:
+        tags['label_color'] = 'navy'
     if 'profile color' not in tags:
         tags['profile color'] = 'navy'
     if 'ring color' not in tags:
         tags['ring color'] = 'red'
-    if 'label size' not in tags:
-        tags['label size'] = 10
+    if 'label_size' not in tags:
+        tags['label_size'] = 10
     if 'profile height' not in tags:
         tags['profile height'] = 5
-    if 'plot scalebar' not in tags:
-        tags['plot scalebar'] = False
+    if 'plot_scalebar' not in tags:
+        tags['plot_scalebar'] = False
+        
     fg, ax = plt.subplots(1, 1)
 
-    ####
+    # ###
     # plot arcs of the rings
-    ####
+    # ###
     for i in range(len(d)):
         pac = patches.Arc((0, 0), d[i] * 2, d[i] * 2, angle=0, theta1=45, theta2=360, color=tags['ring color'])
         ax.add_patch(pac)
@@ -350,22 +392,14 @@ def plotRingPattern(tags, grey=False):
     ####
     # show image in background
     ####
-    if 'plot image' in tags:
-        l = -tags['plot image FOV'] / 2 + tags['plot shift x']
-        r = tags['plot image FOV'] / 2 + tags['plot shift x']
-        t = -tags['plot image FOV'] / 2 + tags['plot shift y']
-        b = tags['plot image FOV'] / 2 + tags['plot shift y']
-        if 'plot image FOV Y' in tags:
-            t = -tags['plot image FOV Y'] / 2 + tags['plot shift y']
-            b = tags['plot image FOV Y'] / 2 + tags['plot shift y']
-
-        plt.imshow(tags['plot image'], extent=(l, r, t, b), cmap='gray')
+    if plot_diffraction_pattern:
+        plt.imshow(diffraction_pattern, extent=diffraction_pattern.get_extent(), cmap='gray')
 
     ax.set_aspect("equal")
 
     # fg.canvas.draw()
 
-    if tags['plot scalebar']:
+    if tags['plot_scalebar']:
         def f(axis):
             l = axis.get_majorticklocs()
             return len(l) > 1 and (l[1] - l[0])
@@ -379,9 +413,9 @@ def plotRingPattern(tags, grey=False):
         ax.add_artist(scalebar)
         ax.axis('off')
 
-    #####
+    # ####
     # plot profile
-    #####
+    # ####
 
     y = tags['Ring_Pattern']['profile_y']
     y = y / y.max() * tags['profile height']
@@ -390,32 +424,65 @@ def plotRingPattern(tags, grey=False):
 
     ax.plot([0, x[-1]], [0, 0], c=tags['profile color'])
 
-    if 'experimental profile y' in tags:
-        yy = tags['experimental profile y']
+    if 'experimental profile_y' in tags:
+        yy = tags['experimental profile_y']
         yy = yy / yy.max() * tags['profile height']
-        xx = tags['experimental profile x']
+        xx = tags['experimental profile_x']
         ax.plot(xx, yy, c=tags['experimental profile color'])
 
-    if 'plot image FOV' in tags:
-        max_d = tags['plot image FOV'] / 2 + tags['plot shift x']
+    if 'plot_image_FOV' in tags:
+        max_d = tags['plot_image_FOV'] / 2 + tags['plot_shift_x']
     else:
         max_d = d.max()
     for i in range(len(d)):
         if d[i] < max_d:
-            plt.text(d[i] - .2, -.5, label[i], fontsize=tags['label size'], color=tags['label color'], rotation=90)
+            plt.text(d[i] - .2, -.5, label[i], fontsize=tags['label_size'], color=tags['label_color'], rotation=90)
 
-    if 'plot FOV' in tags:
-        l = -tags['plot FOV'] / 2
-        r = tags['plot FOV'] / 2
-        t = -tags['plot FOV'] / 2
-        b = tags['plot FOV'] / 2
+    if 'plot_FOV' in tags:
+        l = -tags['plot_FOV'] / 2
+        r = tags['plot_FOV'] / 2
+        t = -tags['plot_FOV'] / 2
+        b = tags['plot_FOV'] / 2
         plt.xlim(l, r)
         plt.ylim(t, b)
 
+
     fg.show()
+    return fg
 
 
-def plot_diffraction_pattern(tagsD, grey=False):
+def plot_diffraction_pattern(atoms, diffraction_pattern=None, grey=False):
+    """
+    Plot of spot diffraction pattern with matplotlib
+    
+    Parameters
+    ----------
+    info: dictionary or ase.Atoms object
+        information stored as dictionary either directly or in info attribute of ase.Atoms object
+    grey: bool
+        ploting in greyscale if True
+        
+    Returns
+    -------
+    fig: matplotlib figure
+        refereence to matplolib figure
+    """
+    
+    if isinstance(atoms, dict):
+        tagsD = atoms
+        
+    elif  isinstance(atoms, ase.Atoms):
+        if 'diffraction' in atoms.info:
+            tagsD = atoms.info['diffraction']
+            plot_diffraction_pattern = True
+        else:
+            raise TypeError('Diffraction information must be in info dictionary of ase.Atoms object')
+    else:
+        raise TypeError('Diffraction info must be in ase.Atoms objec or dictionary form')
+        
+    if 'output' not in atoms.info:
+        return
+    
     # Get information from dictionary
     HOLZ = tagsD['HOLZ']
     ZOLZ = tagsD['allowed']['ZOLZ']
@@ -426,15 +493,15 @@ def plot_diffraction_pattern(tagsD, grey=False):
     label = tagsD['allowed']['label']
     hkl_label = tagsD['allowed']['hkl']
 
-    angle = np.radians(tagsD['plot rotation'])  # mrad
+    angle = np.radians(atoms.info['output']['plot_rotation'])  # mrad
     c = np.cos(angle)
     s = np.sin(angle)
     r_mat = np.array([[c, -s, 0], [s, c, 0], [0, 0, 1]])
 
     # HOLZ and Kikuchi lines coordinates in Hough space
     LC = tagsD['Laue_circle']
-    gd = np.dot(tagsD['HOLZ']['g deficient'] + LC, r_mat)
-    ge = np.dot(tagsD['HOLZ']['g excess'] + LC, r_mat)
+    gd = np.dot(tagsD['HOLZ']['g_deficient'] + LC, r_mat)
+    ge = np.dot(tagsD['HOLZ']['g_excess'] + LC, r_mat)
     points = np.dot(tagsD['allowed']['g'] + LC, r_mat)
 
     theta = tagsD['HOLZ']['theta'] + angle
@@ -446,7 +513,7 @@ def plot_diffraction_pattern(tagsD, grey=False):
     else:
         intensity = tagsD['allowed']['intensities']
 
-    radius = tagsD['convergence_angle_nm-1']
+    radius = atoms.info['output']['convergence_angle_A-1']
 
     if radius < 0.1:
         radiusI = 2
@@ -473,48 +540,40 @@ def plot_diffraction_pattern(tagsD, grey=False):
     Kxm = gd[:, 0] - maxlength * np.cos(np.pi - theta)
     Kym = gd[:, 1] - maxlength * np.sin(np.pi - theta)
 
-    if tagsD['linewidth Kikuchi'] < 0:
+    if atoms.info['output']['linewidth_Kikuchi'] < 0:
         if len(intensity[ZOLZ]) > 0:
             intensity_Kikuchi = intensity * 4. / intensity[ZOLZ].max()
         else:
             intensity_Kikuchi = intensity
     else:
-        intensity_Kikuchi = np.ones(len(intensity)) * tagsD['linewidth Kikuchi']
+        intensity_Kikuchi = np.ones(len(intensity)) * atoms.info['output']['linewidth_Kikuchi']
 
-    if tagsD['linewidth HOLZ'] < 0:
+    if atoms.info['output']['linewidth_HOLZ'] < 0:
         intensity_HOLZ = np.log(intensity + 1)
 
         if tagsD['HOLZ']['HOLZ'].any():
             pass  # intensity_HOLZ = intensity/intensity[tagsD['HOLZ']['HOLZ']].max()*4.
     else:
-        intensity_HOLZ = np.ones(len(intensity)) * tagsD['linewidth HOLZ']
+        intensity_HOLZ = np.ones(len(intensity)) * atoms.info['output']['linewidth_HOLZ']
 
-    ########
+    # #######
     # Plot #
-    ########
-    cms = mpl.cm
+    # #######
+    # cms = mpl.cm
     # cm = cms.plasma#jet#, cms.gray, cms.autumn]
-    cm = plt.get_cmap(tagsD['color map'])
+    cm = plt.get_cmap(atoms.info['output']['color_map'])
 
     # fig = plt.figure()
     fig = plt.figure()
 
     ax = plt.gca()
-    if 'background' not in tagsD:
-        tagsD['background'] = None
-    if tagsD['background'] is not None:
-        ax.set_facecolor(tagsD['background'])
+    if 'background' not in atoms.info['output']:
+        atoms.info['output']['background'] = None
+    if atoms.info['output']['background'] is not None:
+        ax.set_facecolor(atoms.info['output']['background'])
 
-    if 'plot image' in tagsD:
-        l = -tagsD['plot image FOV'] / 2 + tagsD['plot shift x']
-        r = tagsD['plot image FOV'] / 2 + tagsD['plot shift x']
-        t = -tagsD['plot image FOV'] / 2 + tagsD['plot shift y']
-        b = tagsD['plot image FOV'] / 2 + tagsD['plot shift y']
-        if 'plot image FOV Y' in tagsD:
-            t = -tagsD['plot image FOV Y'] / 2 + tagsD['plot shift y']
-            b = tagsD['plot image FOV Y'] / 2 + tagsD['plot shift y']
-
-        plt.imshow(tagsD['plot image'], extent=(l, r, t, b))
+    if diffraction_pattern is not None:
+        plt.imshow(diffraction_pattern, extent=diffraction_pattern.get_extent(), cmap='gray')
 
     ix = np.argsort((points ** 2).sum(axis=1))
     p = points[ix]
@@ -549,21 +608,20 @@ def plot_diffraction_pattern(tagsD, grey=False):
             print('Reflection: [%1d,%1d,%1d]' % (h, k, l))
 
     for i in range(int(Laue_Zone.max()) + 1):
-        if i < len(tagsD['color Laue Zones']):
-            Lauecolor.append(tagsD['color Laue Zones'][i])
+        if i < len(atoms.info['output']['color_Laue_Zones']):
+            Lauecolor.append(atoms.info['output']['color_Laue_Zones'][i])
         else:
-            Lauecolor.append(tagsD['color Laue Zones'][-1])
+            Lauecolor.append(atoms.info['output']['color_Laue_Zones'][-1])
 
-    if 'plot_labels' not in tagsD:
-        tagsD['plot_labels'] = True
-    if tagsD['plot reflections']:
-        print(radius)
-        if radius < 0.1:
-            if tagsD['color reflections'] == 'intensity':
+    if 'plot_labels' not in atoms.info['output']:
+        atoms.info['output']['plot_labels'] = True
+    if atoms.info['output']['plot_reflections']:
+        if radius < 0.01:
+            if atoms.info['output']['color_reflections'] == 'intensity':
                 for i in range(len(points)):
                     ax.scatter(points[i, 0], points[i, 1], c=np.log(intensity[i] + 1), cmap=cm, s=100)
 
-                    if tagsD['plot_labels']:
+                    if atoms.info['output']['plot_labels']:
                             plt.text(points[i, 0], points[i, 1], label[i], fontsize=10)
                     #print(label[i])
 
@@ -571,17 +629,17 @@ def plot_diffraction_pattern(tagsD, grey=False):
                 for i in range(len(Laue_Zone)):
                     color = Lauecolor[int(Laue_Zone[i])]
                     ax.scatter(points[i, 0], points[i, 1], c=color, cmap=cm, s=100)
-                    if tagsD['plot_labels']:
+                    if atoms.info['output']['plot_labels']:
                         plt.text(points[i, 0], points[i, 1], label[i], fontsize=8)
                     #print(label[i])
 
-            ax.scatter(0, 0, c=tagsD['color zero'], s=100)
-            radius = 2
+            ax.scatter(0, 0, c=atoms.info['output']['color_zero'], s=100)
+            radius = .2
         else:
             ix = np.argsort((points ** 2).sum(axis=1))
             p = points[ix]
             inten = intensity[ix]
-            if tagsD['color reflections'] == 'intensity':
+            if atoms.info['output']['color_reflections'] == 'intensity':
                 circles(p[:, 0], p[:, 1], s=radius, c=np.log(inten + 1), cmap=cm, alpha=0.9, edgecolor='', picker=5)  #
             else:
                 for i in range(len(Laue_Zone)):
@@ -589,40 +647,51 @@ def plot_diffraction_pattern(tagsD, grey=False):
                     circles(p[i, 0], p[i, 1], s=radius, c=color, cmap=cm, alpha=0.9, edgecolor='', picker=5)  #
                     plt.text(points[i, 0], points[i, 1], label[i], fontsize=8)
 
-    k = 0
-    if tagsD['plot HOLZ']:
-        for i in range(len(Hxp)):
+    if 'plot_dynamically_allowed' not in atoms.info['output']:
+        atoms.info['output']['plot_dynamically_allowed'] = False
+    if atoms.info['output']['plot_dynamically_allowed']:
+        dyn_allowed = tagsD['dynamical_allowed']['g']
+        dyn_label = tagsD['dynamical_allowed']['hkl']
+        
+        color = Lauecolor[0]
+        ax.scatter(dyn_allowed[:, 0], dyn_allowed[:, 1], c=color, alpha = 0.4, s=70)
+        if atoms.info['output']['plot_labels']:
+            plt.text(dyn_allowed[i, 0], dyn_allowed[i, 1], dyn_label[i], fontsize=8)
 
+    k = 0
+    if atoms.info['output']['plot_HOLZ']:
+        for i in range(len(Hxp)):
+        
             if tagsD['HOLZ']['HOLZ'][i]:
                 color = Lauecolor[int(Laue_Zone[i])]
-                if tagsD['plot HOLZ']:
+                if tagsD['plot_HOLZ']:
                     # plot HOLZ lines
                     line, = plt.plot((Hxp[i], Hxm[i]), (Hyp[i], Hym[i]), c=color, linewidth=intensity_HOLZ[i], picker=5)
-                    if tagsD['label HOLZ']:  ## Add indices
+                    if tagsD['label_HOLZ']:  ## Add indices
                         plt.text(Hxp[i], Hyp[i], label[i], fontsize=8)
                     lineLabel.append(hkl_label[i])
                     # print(i, hkl_label[i], intensity_HOLZ[i])
 
-                if tagsD['plot HOLZ excess']:
+                if tagsD['plot_HOLZ_excess']:
                     line, = plt.plot((Exp[i], Exm[i]), (Eyp[i], Eym[i]), c=color, linewidth=intensity_HOLZ[i])
                     lineLabel.append(hkl_label[i])
 
-                    if tagsD['label HOLZ']:  ## Add indices
+                    if tagsD['label_HOLZ']:  ## Add indices
                         plt.text(Exp[i], Eyp[i], label[i], fontsize=8)
 
 
             else:
                 # Plot Kikuchi lines
-                if tagsD['plot Kikuchi']:
-                    line, = plt.plot((Kxp[i], Kxm[i]), (Kyp[i], Kym[i]), c=tagsD['color Kikuchi'],
+                if tagsD['plot_Kikuchi']:
+                    line, = plt.plot((Kxp[i], Kxm[i]), (Kyp[i], Kym[i]), c=tagsD['color_Kikuchi'],
                                      linewidth=intensity_Kikuchi[i] / 10, picker=5)
-                    if tagsD['label Kikuchi']:  # Add indices
+                    if tagsD['label_Kikuchi']:  # Add indices
 
-                        plt.text(Kxp[i], Kyp[i], label[i], fontsize=tagsD['label size'], color=tagsD['label color'])
+                        plt.text(Kxp[i], Kyp[i], label[i], fontsize=tagsD['label_size'], color=tagsD['label_color'])
                     lineLabel.append(hkl_label[i])
-    elif tagsD['plot Kikuchi']:
+    elif atoms.info['output']['plot_Kikuchi']:
         # Beginning and ends of Kikuchi lines
-        maxlength = tagsD['plot FOV']
+        maxlength = atoms.info['experimental']['plot_FOV']
         gd = tagsD['Kikuchi']['min dist']
         theta = tagsD['Kikuchi']['theta']
         Kxp = gd[:, 0] + maxlength * np.cos(np.pi - theta)
@@ -630,27 +699,27 @@ def plot_diffraction_pattern(tagsD, grey=False):
         Kxm = gd[:, 0] - maxlength * np.cos(np.pi - theta)
         Kym = gd[:, 1] - maxlength * np.sin(np.pi - theta)
         for i in range(len(Kxp)):
-            line, = plt.plot((Kxp[i], Kxm[i]), (Kyp[i], Kym[i]), c=tagsD['color Kikuchi'], linewidth=2)
+            line, = plt.plot((Kxp[i], Kxm[i]), (Kyp[i], Kym[i]), c=atoms.info['output']['color_Kikuchi'], linewidth=2)
 
     def format_coord(x, y):
         return labelP + 'x=%1.4f, y=%1.4f' % (x, y)
 
     ax.format_coord = format_coord
 
-    if not (tagsD['color ring zero'] == 'None'):
-        ring = plt.Circle((0, 0), radius, color=tagsD['color ring zero'], fill=False, linewidth=2)
+    if not (atoms.info['output']['color_ring_zero'] == 'None'):
+        ring = plt.Circle((0, 0), radius, color=atoms.info['output']['color_ring_zero'], fill=False, linewidth=2)
         ax.add_artist(ring)
         # print(ring)
-    if not tagsD['color zero'] == 'None':
-        circle = plt.Circle((0, 0), radius, color=tagsD['color zero'], linewidth=2)
+    if not atoms.info['output']['color_zero'] == 'None':
+        circle = plt.Circle((0, 0), radius, color=atoms.info['output']['color_zero'], linewidth=2)
         ax.add_artist(circle)
 
     plt.axis('equal')
-    if 'plot FOV' in tagsD:
-        l = -tagsD['plot FOV'] / 2
-        r = tagsD['plot FOV'] / 2
-        t = -tagsD['plot FOV'] / 2
-        b = tagsD['plot FOV'] / 2
+    if 'plot_FOV' in tagsD:
+        l = -tagsD['plot_FOV'] / 2
+        r = tagsD['plot_FOV'] / 2
+        t = -tagsD['plot_FOV'] / 2
+        b = tagsD['plot_FOV'] / 2
         plt.xlim(l, r)
         plt.ylim(t, b)
 
