@@ -1,6 +1,5 @@
 """
-KinsCat
-Kinematic Scattering Theory
+kinematic_scattering
 Copyright by Gerd Duscher
 
 The University of Tennessee, Knoxville
@@ -17,12 +16,12 @@ Sources:
        Appendix C
 
 Units:
-    everything is in SI units, except length is given in nm.
+    everything is in SI units, except length which is given in Angstrom.
 
 Usage:
     See the notebooks for examples of these routines
 
-All the input and output is done through a dictionary
+All the input and output is done through a ase.Atoms object and the dictionary in the info attribute
 """
 
 # numerical packages used
@@ -37,21 +36,11 @@ import pyTEMlib.file_tools as ft
 from pyTEMlib.crystal_tools import *
 from pyTEMlib.diffraction_plot import *
 
-_version_ = 0.6
+_version_ = "0.2022.1.0"
 
-print('Using kinematic_scattering library version ', _version_, ' by G.Duscher')
-_spglib_present = True
-try:
-    import spglib
-except ModuleNotFoundError:
-    _spglib_present = False
+print('Using kinematic_scattering library version {_version_ }  by G.Duscher')
 
-if _spglib_present:
-    print('Symmetry functions of spglib enabled')
-else:
-    print('spglib not installed; Symmetry functions of spglib disabled')
-
-inputKeys = ['unit_cell', 'elements', 'base', 'acceleration_voltage_V', 'zone_hkl', 'Sg_max', 'hkl_max']
+inputKeys = ['acceleration_voltage_V', 'zone_hkl', 'Sg_max', 'hkl_max']
 optional_inputKeys = ['crystal', 'lattice_parameter_nm', 'convergence_angle_mrad', 'mistilt', 'thickness',
                       'dynamic correction', 'dynamic correction K0']
 
@@ -81,7 +70,15 @@ def Zuo_fig_3_18(verbose=True):
 
     This input acts as an example as well as a reference
 
+    Parameters:
+    -----------
+    verbose: boolean:
+        optional to see output
     Returns:
+    -------
+        atoms: ase.Atoms
+            Silicon crystal structure
+        e
         dictionary: tags is the dictionary of all input and output parameter needed to reproduce that figure.
     """
 
@@ -89,122 +86,65 @@ def Zuo_fig_3_18(verbose=True):
     # Create Silicon structure (Could be produced with Silicon routine)
     if verbose:
         print('Sample Input for Figure 3.18 in Zuo and Spence \"Advanced TEM\", 2017')
-    tags = {'crystal_name': 'Silicon'}
-    if verbose:
-        print('tags[\'crystal\'] = ', tags['crystal_name'])
-    a = 0.514  # nm
-
-    tags['lattice_parameter'] = a
-    if verbose:
-        print('tags[\'lattice_parameter\'] =', tags['lattice_parameter'])
-    tags['unit_cell'] = [[a, 0, 0], [0, a, 0], [0, 0, a]]
-    if verbose:
-        print('tags[\'unit_cell\'] =', tags['unit_cell'])
-    tags['elements'] = list(itertools.repeat('Si', 8))
-    if verbose:
-        print('tags[\'atoms\'] =', tags['elements'])
-    base = [(0., 0., 0.), (0.5, 0.0, 0.5), (0.5, 0.5, 0.), (0., 0.5, 0.5)]
-    tags['base'] = np.array(base + (np.array(base) + (.25, .25, .25)).tolist())
-    if verbose:
-        print('tags[\'base\'] =', tags['base'])
+    import ase
+    import ase.build
+    a = 5.14  # A
+    atoms = ase.build.bulk('Si', 'diamond', a=a, cubic=True)
 
+    experiment = {'acceleration_voltage_V': 99.2 * 1000.0,  # V
+                  'convergence_angle_mrad': 7.15,  # mrad;
+                  'zone_hkl': np.array([-2, 2, 1]),
+                  'mistilt': np.array([0, 0, 0]),  # mistilt in degrees
+                  'Sg_max': .03,  # 1/A  maximum allowed excitation error
+                  'hkl_max': 9  # Highest evaluated Miller indices
+                  }
     # Define Experimental Conditions
-    tags['acceleration_voltage_V'] = 99.2 * 1000.0  # V
     if verbose:
-        print('tags[\'acceleration_voltage_V\'] =', tags['acceleration_voltage_V'])
+        print('###########################')
+        print('# Experimental Conditions #')
+        print('###########################')
 
-    tags['convergence_angle_mrad'] = 7.15  # mrad;  0 is parallel illumination
-    if verbose:
-        print('tags[\'convergence_angle_mrad\'] =', tags['convergence_angle_mrad'])
-
-    tags['zone_hkl'] = np.array([-2, 2, 1])  # incident neares zone axis: defines Laue Zones!!!!
-    if verbose:
-        print('tags[\'zone_hkl\'] =', tags['zone_hkl'])
-
-    tags['mistilt'] = np.array([0, 0, 0])  # mistilt in degrees
-    if verbose:
-        print('tags[\'mistilt\'] =', tags['mistilt'])
-
-    # Define Simulation Parameters
-    tags['Sg_max'] = .3  # 1/nm  maximum allowed excitation error
-    if verbose:
-        print('tags[\'Sg_max\'] =', tags['Sg_max'])
-
-    tags['hkl_max'] = 9  # Highest evaluated Miller indices
-    if verbose:
-        print('tags[\'hkl_max\'] =', tags['hkl_max'])
+        for key, value in experiment.items():
+            print(f'tags[\'{key}\'] =', value)
 
         print('##################')
         print('# Output Options #')
         print('##################')
 
     # Output options
-    tags['background'] = 'black'  # 'white'  'grey'
-    if verbose:
-        print('tags[\'background\'] =', tags['background'], '# \'white\',  \'grey\' ')
-    tags['color map'] = 'plasma'
-    if verbose:
-        print('tags[\'color map\'] =', tags['color map'], '#,\'cubehelix\',\'Greys\',\'jet\' ')
+    output = {'background': 'black',  # 'white'  'grey'
+              'color_map': 'plasma',
+              'plot_HOLZ': True,
+              'plot_HOLZ_excess': True,
+              'plot_Kikuchi': True,
+              'plot_reflections': True,
+              'label_HOLZ': False,
+              'label_Kikuchi': False,
+              'label_reflections': False,
+              'label_color': 'black',
+              'label_size': 10,
+              'color_Laue_Zones': ['red', 'blue', 'green', 'blue', 'green'],  # for OLZ give a sequence
+              'color_Kikuchi': 'green',
+              'linewidth_HOLZ': -1,  # -1: linewidth according to intensity (structure factor F^2)
+              'linewidth_Kikuchi': -1,  # -1: linewidth according to intensity (structure factor F^2)
+              'color_reflections': 'intensity',  # 'Laue Zone'
+              'color_zero': 'white',  # 'None', 'white', 'blue'
+              'color_ring_zero': 'None'  # 'Red' #'white' #, 'None'
+              }
 
-    tags['plot HOLZ'] = 1
     if verbose:
-        print('tags[\'plot HOLZ\'] =', tags['plot HOLZ'])
-    tags['plot HOLZ excess'] = 1
-    if verbose:
-        print('tags[\'plot HOLZ excess\'] =', tags['plot HOLZ excess'])
-    tags['plot Kikuchi'] = 1
-    if verbose:
-        print('tags[\'plot Kikuchi\'] =', tags['plot Kikuchi'])
-    tags['plot reflections'] = 1
-    if verbose:
-        print('tags[\'plot reflections\'] =', tags['plot reflections'])
-
-    tags['label HOLZ'] = 0
-    if verbose:
-        print('tags[\'label HOLZ\'] =', tags['label HOLZ'])
-    tags['label Kikuchi'] = 0
-    if verbose:
-        print('tags[\'label Kikuchi\'] =', tags['label Kikuchi'])
-    tags['label reflections'] = 0
-    if verbose:
-        print('tags[\'label reflections\'] =', tags['label reflections'])
-
-    tags['label color'] = 'black'
-    if verbose:
-        print('tags[\'label color\'] =', tags['label color'])
-    tags['label size'] = 10
-    if verbose:
-        print('tags[\'label size\'] =', tags['label size'])
-
-    tags['color Laue Zones'] = ['red', 'blue', 'green', 'blue', 'green']  # for OLZ give a sequence
-    if verbose:
-        print('tags[\'color Laue Zones\'] =', tags['color Laue Zones'], ' #[\'red\', \'blue\', \'lightblue\']')
-
-    tags['color Kikuchi'] = 'green'
-    if verbose:
-        print('tags[\'color Kikuchi\'] =', tags['color Kikuchi'])
-    tags['linewidth HOLZ'] = -1  # -1: linewidth according to intensity (structure factor F^2
-    if verbose:
-        print('tags[\'linewidth HOLZ\'] =', tags['linewidth HOLZ'], '# -1: linewidth according to intensity '
-                                                                    '(structure factor F^2)')
-    tags['linewidth Kikuchi'] = -1  # -1: linewidth according to intensity (structure factor F^2
-    if verbose:
-        print('tags[\'linewidth Kikuchi\'] =', tags['linewidth Kikuchi'], '# -1: linewidth according to intensity '
-                                                                          '(structure factor F^2)')
-
-    tags['color reflections'] = 'intensity'  # 'Laue Zone'
-    if verbose:
-        print('tags[\'color reflections\'] =', tags['color reflections'], '#\'Laue Zone\' ')
-    tags['color zero'] = 'white'  # 'None', 'white', 'blue'
-    if verbose:
-        print('tags[\'color zero\'] =', tags['color zero'], '#\'None\', \'white\', \'blue\' ')
-    tags['color ring zero'] = 'None'  # 'Red' #'white' #, 'None'
-    if verbose:
-        print('tags[\'color ring zero\'] =', tags['color ring zero'], '#\'None\', \'white\', \'Red\' ')
+        for key, value in output.items():
+            print(f'tags[\'{key}\'] =', value)
         print('########################')
         print('# End of Example Input #')
         print('########################\n\n')
-    return tags
+
+    if atoms.info is None:
+        atoms.info = {}
+    atoms.info['experimental'] = experiment
+    atoms.info['output'] = output
+
+    return atoms
 
 
 def zone_mistilt(zone, angles):
@@ -247,46 +187,6 @@ def zone_mistilt(zone, angles):
     return np.dot(np.dot(np.dot(zone, rot_x), rot_y), rot_z)
 
 
-def get_symmetry(unit_cell, base, atoms, verbose=True):
-    """
-    Symmetry analysis with spglib
-
-    spglib must be installed
-    """
-    if _spglib_present:
-        if verbose:
-            print('#####################')
-            print('# Symmetry Analysis #')
-            print('#####################')
-
-        atomic_number = []
-        for i in range(len(atoms)):
-            a = atoms[i]
-            b = base[i]
-            atomic_number.append(electronFF[a]['Z'])
-            if verbose:
-                print(f'{i + 1}: {atomic_number[i]} = {2} : [{base[i][0]:.2f}, {base[i][1]:.2f}, {base[i][2]:.2f}]')
-
-        lattice = (unit_cell, base, atomic_number)
-        spgroup = spglib.get_spacegroup(lattice)
-        sym = spglib.get_symmetry(lattice)
-
-        if verbose:
-            print("  Spacegroup  is %s." % spgroup)
-            print('  Crystal has {0} symmetry operation'.format(sym['rotations'].shape[0]))
-
-        p_lattice, p_positions, p_numbers = spglib.find_primitive(lattice, symprec=1e-5)
-        print("\n########################\n #Basis vectors of primitive Cell:")
-        for i in range(3):
-            print('[{0:.4f}, {1:.4f}, {2:.4f}]'.format(p_lattice[i][0], p_lattice[i][1], p_lattice[i][2]))
-
-        print('There {0} atoms and {1} species in primitive unit cell:'.format(len(p_positions), p_numbers))
-    else:
-        print('spglib is not installed')
-
-    return True
-
-
 def get_metric_tensor(matrix):
     """The metric tensor of the lattice."""
     metric_tensor2 = np.dot(matrix, matrix.T)
@@ -302,21 +202,23 @@ def vector_norm(g):
     return np.sqrt(g[:, 0] ** 2 + g[:, 1] ** 2 + g[:, 2] ** 2)
 
 
-def get_wavelength(e0):
+def get_wavelength(acceleration_voltage):
     """
-    Calculates the relativistic corrected de Broglie wavelength of an electron in nm
+    Calculates the relativistic corrected de Broglie wavelength of an electron in Angstrom
 
-    Input:
-    ------
+    Parameter:
+    ---------
+    acceleration_voltage: float
         acceleration voltage in volt
-    Output:
+    Returns:
     -------
-        wave length in nm
+    wavelength: float
+        wave length in Angstrom
     """
-    if not isinstance(e0, (int, float)):
+    if not isinstance(acceleration_voltage, (int, float)):
         raise TypeError('Acceleration voltage has to be a real number')
-    eV = const.e * e0
-    return const.h/np.sqrt(2*const.m_e*eV*(1+eV/(2*const.m_e*const.c**2)))*10**9
+    eU = const.e * acceleration_voltage
+    return const.h/np.sqrt(2*const.m_e*eU*(1+eU/(2*const.m_e*const.c**2)))*10**10
 
 
 def find_nearest_zone_axis(tags):
@@ -366,7 +268,7 @@ def find_nearest_zone_axis(tags):
 
 
 def find_angles(zone):
-    """Microscope stage cooordinates of zone"""
+    """Microscope stage coordinates of zone"""
 
     # rotation around y-axis
     r = np.sqrt(zone[1] ** 2 + zone[2] ** 2)
@@ -394,10 +296,10 @@ def stage_rotation_matrix(alpha, beta):
     return np.dot(rot_x, rot_y)
 
 
-###################
+# ##################
 # Determine rotation matrix to tilt zone axis onto z-axis
 # We determine spherical coordinates to do that
-###################
+# ##################
 
 def get_rotation_matrix(tags):
     """zone axis in global coordinate system"""
@@ -457,18 +359,20 @@ def get_rotation_matrix(tags):
     return rotation_matrix
 
 
-def check_sanity(tags, verbose_level=0):
+def check_sanity(atoms, verbose_level=0):
     """
     Check sanity of input parameters
     """
     stop = False
-    for key in ['unit_cell', 'base', 'elements', 'acceleration_voltage_V']:
+    output = atoms.info['output']
+    tags = atoms.info['experimental']
+    for key in ['acceleration_voltage_V']:
         if key not in tags:
             print(f'Necessary parameter {key} not defined')
             stop = True
-    if 'SpotPattern' not in tags:
-        tags['SpotPattern'] = False
-    if tags['SpotPattern']:
+    if 'SpotPattern' not in output:
+        output['SpotPattern'] = False
+    if output['SpotPattern']:
         if 'zone_hkl' not in tags:
             print(' No zone_hkl defined')
             stop = True
@@ -487,13 +391,9 @@ def check_sanity(tags, verbose_level=0):
     # Check optional input
     ############################################
 
-    if 'crystal' not in tags:
-        tags['crystal'] = 'undefined'
-        if verbose_level > 0:
-            print('Setting undefined input: tags[\'crystal\'] = \'undefined\'')
-    if tags['SpotPattern']:
+    if output['SpotPattern']:
         if 'mistilt alpha degree' not in tags:
-            # mistilt is in microcope coordinates
+            # mistilt is in microscope coordinates
             tags['mistilt alpha'] = tags['mistilt alpha degree'] = 0.0
             if verbose_level > 0:
                 print('Setting undefined input:  tags[\'mistilt alpha\'] = 0.0 ')
@@ -532,9 +432,9 @@ def scattering_matrix(tags, verbose_level=1):
     """ Scattering matrix"""
     if not check_sanity(tags, verbose_level):
         return
-    ###
+    # ##
     # Pair distribution Function
-    ###
+    # ##
     unit_cell = np.array(tags['unit_cell'])
     base = tags['base']
 
@@ -566,14 +466,14 @@ def scattering_matrix(tags, verbose_level=1):
         plt.show()
 
 
-def ring_pattern_calculation(tags, verbose=False):
+def ring_pattern_calculation(atoms, verbose=False):
     """
     Calculate the ring diffraction pattern of a crystal structure
 
     Parameters
     ----------
-    tags: dict
-        dictionary of crystal structure
+    atoms: Crystal
+        crystal structure
     verbose: verbose print outs
         set to False
     Returns
@@ -583,15 +483,15 @@ def ring_pattern_calculation(tags, verbose=False):
     """
     
     # Check sanity
-    tags['SpotPattern'] = False
-    if not check_sanity(tags, verbose):
+    if not check_sanity(atoms, verbose):
         return
 
+    tags = atoms.info['experimental']
     # wavelength
-    tags['wave_length_nm'] = get_wavelength(tags['acceleration_voltage_V'])
+    tags['wave_length'] = get_wavelength(tags['acceleration_voltage_V'])
 
     #  volume of unit_cell
-    unit_cell = np.array(tags['unit_cell'])
+    unit_cell = atoms.cell.array
     metric_tensor = get_metric_tensor(unit_cell)  # converts hkl to g vectors and back
     tags['metric_tensor'] = metric_tensor
     # volume_unit_cell = np.sqrt(np.linalg.det(metric_tensor))
@@ -599,7 +499,7 @@ def ring_pattern_calculation(tags, verbose=False):
     # reciprocal_unit_cell
     
     # We use the linear algebra package of numpy to invert the unit_cell "matrix"
-    reciprocal_unit_cell = np.linalg.inv(unit_cell).T  # transposed of inverted unit_cell
+    reciprocal_unit_cell = atoms.cell.reciprocal()  # np.linalg.inv(unit_cell).T  # transposed of inverted unit_cell
     tags['reciprocal_unit_cell'] = reciprocal_unit_cell
     # inverse_metric_tensor = get_metric_tensor(reciprocal_unit_cell)
 
@@ -621,9 +521,9 @@ def ring_pattern_calculation(tags, verbose=False):
     structure_factors = []
     for j in range(len(g_hkl)):
         F = 0
-        for b in range(len(tags['base'])):
-            f = feq(tags['elements'][b], np.linalg.norm(g_hkl[j]))
-            F += f * np.exp(-2 * np.pi * 1j * (hkl[j] * tags['base'][b]).sum())
+        for b in range(len(atoms)):
+            f = feq(atoms[b].symbol, np.linalg.norm(g_hkl[j]))
+            F += f * np.exp(-2 * np.pi * 1j * (hkl[j] * atoms.get_scaled_positions()[b]).sum())
 
         structure_factors.append(F)
 
@@ -670,21 +570,21 @@ def ring_pattern_calculation(tags, verbose=False):
         print('\n\n [hkl]  \t 1/d [1/nm] \t d [nm] \t F^2 ')
         for i in range(len(unique)):
             print(' {0} \t {1:.2f} \t         {2:.4f} \t {3:.2f} '
-                  .format(reflections_m[i], unique[i], 1 / unique[i], np.real(reflections_F[i]) ** 2))
+                  .format(reflections_m[i], unique[i]*10., 1 / unique[i]/10., np.real(reflections_F[i]) ** 2))
 
-    tags['Ring_Pattern'] = {}
-    tags['Ring_Pattern']['allowed'] = {}
-    tags['Ring_Pattern']['allowed']['hkl'] = reflections_m
-    tags['Ring_Pattern']['allowed']['g norm'] = unique
-    tags['Ring_Pattern']['allowed']['structure factor'] = reflections_F
-    tags['Ring_Pattern']['allowed']['multiplicity'] = counts
+    atoms.info['Ring_Pattern'] = {}
+    atoms.info['Ring_Pattern']['allowed'] = {}
+    atoms.info['Ring_Pattern']['allowed']['hkl'] = reflections_m
+    atoms.info['Ring_Pattern']['allowed']['g norm'] = unique
+    atoms.info['Ring_Pattern']['allowed']['structure factor'] = reflections_F
+    atoms.info['Ring_Pattern']['allowed']['multiplicity'] = counts
 
-    tags['Ring_Pattern']['profile_x'] = np.linspace(0, unique.max(), 2048)
-    step_size = tags['Ring_Pattern']['profile_x'][1]
+    atoms.info['Ring_Pattern']['profile_x'] = np.linspace(0, unique.max(), 2048)
+    step_size = atoms.info['Ring_Pattern']['profile_x'][1]
     intensity = np.zeros(2048)
     x_index = [(unique / step_size + 0.5).astype(np.int)]
     intensity[x_index] = np.array(np.real(reflections_F)) * np.array(np.real(reflections_F))
-    tags['Ring_Pattern']['profile_y delta'] = intensity
+    atoms.info['Ring_Pattern']['profile_y delta'] = intensity
 
     def gaussian(xx, pp):
         s1 = pp[2] / 2.3548
@@ -699,16 +599,15 @@ def ring_pattern_calculation(tags, verbose=False):
 
             gauss = gaussian(x, p)
             intensity = np.convolve(np.array(intensity), np.array(gauss), mode='same')
-    tags['Ring_Pattern']['profile_y'] = intensity
+    atoms.info['Ring_Pattern']['profile_y'] = intensity
 
     # Make pretty labels
     hkl_allowed = reflections_m
     hkl_label = make_pretty_labels(hkl_allowed)
-    tags['Ring_Pattern']['allowed']['label'] = hkl_label
+    atoms.info['Ring_Pattern']['allowed']['label'] = hkl_label
 
 
-
-def kinematic_scattering(tags, verbose=False):
+def kinematic_scattering(atoms, verbose=False):
     """
         All kinematic scattering calculation
 
@@ -716,10 +615,9 @@ def kinematic_scattering(tags, verbose=False):
 
         Parameters
         ----------
-        tags: dict
-            dictionary with crystal structure:
-            'unit_cell', 'base' 'elements'
-            and with experimental parameters:
+        atoms: ase.Atoms
+            object with crystal structure:
+            and with experimental parameters in info attribute:
             'acceleration_voltage_V', 'zone_hkl', 'Sg_max', 'hkl_max'
             Optional parameters are:
             'mistilt', convergence_angle_mrad', and 'crystal_name'
@@ -729,14 +627,14 @@ def kinematic_scattering(tags, verbose=False):
 
         Returns
         -------
-        dict:
-            There are three sub_dictionaries:
+        ato,s:
+            There are three sub_dictionaries in info attribute:
             ['allowed'], ['forbidden'], and ['HOLZ']
             ['allowed'] and ['forbidden'] dictionaries contain:
                 ['Sg'], ['hkl'], ['g'], ['structure factor'], ['intensities'],
                 ['ZOLZ'], ['FOLZ'], ['SOLZ'], ['HOLZ'], ['HHOLZ'], ['label'], and ['Laue_zone']
             the ['HOLZ'] dictionary contains:
-                ['slope'], ['distance'], ['theta'], ['g deficient'], ['g excess'], ['hkl'], ['intensities'],
+                ['slope'], ['distance'], ['theta'], ['g_deficient'], ['g_excess'], ['hkl'], ['intensities'],
                 ['ZOLZ'], ['FOLZ'], ['SOLZ'], ['HOLZ'], and  ['HHOLZ']
             Please note that the Kikuchi lines are the HOLZ lines of ZOLZ
 
@@ -746,25 +644,38 @@ def kinematic_scattering(tags, verbose=False):
     """
 
     # Check sanity
-    tags['SpotPattern'] = True
-    if not check_sanity(tags):
+    if atoms.info is None:
+        atoms.info = {'output': {}, 'experimental': {}}
+    elif 'output' in atoms.info:
+        output = atoms.info['output']
+    else:
+        output = atoms.info['output'] = {}
+
+    output['SpotPattern'] = True
+
+    if 'experimental' not in atoms.info:
+        tags = atoms.info['experimental'] = {}
+
+    if not check_sanity(atoms):
         print('Input is not complete, stopping')
         print('Try \'example()\' for example input')
         return
 
+    tags = atoms.info['experimental']
+
     # wavelength
-    tags['wave_length_nm'] = get_wavelength(tags['acceleration_voltage_V'])
+    tags['wave_length'] = get_wavelength(tags['acceleration_voltage_V'])
 
     #  volume of unit_cell
-    unit_cell = np.array(tags['unit_cell'])
+    unit_cell = atoms.cell.array
     metric_tensor = get_metric_tensor(unit_cell)  # converts hkl to g vectors and back
     tags['metric_tensor'] = metric_tensor
-    volume_unit_cell = np.sqrt(np.linalg.det(metric_tensor))
+    volume_unit_cell = atoms.cell.volume
 
     # reciprocal_unit_cell
 
     # We use the linear algebra package of numpy to invert the unit_cell "matrix"
-    reciprocal_unit_cell = np.linalg.inv(unit_cell).T  # transposed of inverted unit_cell
+    reciprocal_unit_cell = atoms.cell.reciprocal()  # np.linalg.inv(unit_cell).T  # transposed of inverted unit_cell
     tags['reciprocal_unit_cell'] = reciprocal_unit_cell
     inverse_metric_tensor = get_metric_tensor(reciprocal_unit_cell)
 
@@ -776,15 +687,15 @@ def kinematic_scattering(tags, verbose=False):
     # Incident wave vector k0 in vacuum and material
     ############################################
 
-    ratio = (1 + 1.9569341 * tags['acceleration_voltage_V']) / (np.pi * volume_unit_cell * 1000.)
+    ratio = (1 + 1.9569341 * tags['acceleration_voltage_V']) / (np.pi * volume_unit_cell)
 
     u0 = 0  # in (Ang)
     # atom form factor of zero reflection angle is the inner potential in 1/A
-    for i in range(len(tags['elements'])):
-        u0 += feq(tags['elements'][i], 0)
+    for i in range(len(atoms)):
+        u0 += feq(atoms[i].symbol, 0)
 
     # Conversion of inner potential to Volts
-    u0 = u0 * ratio / 100.0  # inner potential in 1/nm^2
+    u0 = u0 * ratio   # inner potential in 1/Ang^2
 
     scattering_factor_to_volts = (const.h ** 2) * (1e10 ** 2) / (2 * np.pi * const.m_e * const.e) * volume_unit_cell
 
@@ -793,20 +704,20 @@ def kinematic_scattering(tags, verbose=False):
         print('The inner potential is {0:.1f}V'.format(u0))
 
     # Calculating incident wave vector magnitude 'k0' in material
-    wl = tags['wave_length_nm']
+    wl = tags['wave_length']
     tags['incident_wave_vector_vacuum'] = 1 / wl
 
-    k0 = tags['incident_wave_vector'] = np.sqrt(1 / wl ** 2 + u0)  # 1/nm
+    k0 = tags['incident_wave_vector'] = np.sqrt(1 / wl ** 2 + u0)  # 1/Ang
 
-    tags['convergence_angle_nm-1'] = k0 * np.sin(tags['convergence_angle_mrad'] / 1000.)
+    tags['convergence_angle_A-1'] = k0 * np.sin(tags['convergence_angle_mrad'] / 1000.)
     if verbose:
         print(f"Using an acceleration voltage of {tags['acceleration_voltage_V']/1000:.1f}kV")
-        print(f'Magnitude of incident wave vector in material: {k0:.1f} 1/nm and in vacuum: {1/wl:.1f} 1/nm')
-        print(f"Which is an wave length of {1 / k0 * 1000.:.3f} pm in the material and {wl * 1000.:.3f} pm "
+        print(f'Magnitude of incident wave vector in material: {k0:.1f} 1/Ang and in vacuum: {1/wl:.1f} 1/Ang')
+        print(f"Which is an wave length of {1 / k0 * 100.:.3f} pm in the material and {wl * 100.:.3f} pm "
               f"in the vacuum")
         print(f"The convergence angle of {tags['convergence_angle_mrad']:.1f}mrad "
-              f"= {tags['convergence_angle_nm-1']:.2f} 1/nm")
-        print(f"Magnitude of incident wave vector in material: {k0:.1f} 1/nm which is a wavelength {1000./k0:.3f} pm")
+              f"= {tags['convergence_angle_A-1']:.2f} 1/A")
+        print(f"Magnitude of incident wave vector in material: {k0:.1f} 1/A which is a wavelength {100/k0:.3f} pm")
 
     # ############
     # Rotate
@@ -816,7 +727,6 @@ def kinematic_scattering(tags, verbose=False):
     rotation_matrix = get_rotation_matrix(tags)
 
     if verbose:
-
         print(f"Rotation alpha {np.rad2deg(tags['y-axis rotation alpha']):.1f} degree, "
               f" beta {np.rad2deg(tags['x-axis rotation beta']):.1f} degree")
         print(f"from zone axis {tags['zone_hkl']}")
@@ -843,7 +753,7 @@ def kinematic_scattering(tags, verbose=False):
     # Find all Miller indices whose reciprocal point lays near the Ewald sphere with radius k0
     # within a maximum excitation error Sg
     hkl_max = tags['hkl_max']
-    Sg_max = tags['Sg_max']  # 1/nm  maximum allowed excitation error
+    Sg_max = tags['Sg_max']  # 1/A  maximum allowed excitation error
 
     h = np.linspace(-hkl_max, hkl_max, 2 * hkl_max + 1)  # all evaluated single Miller Indices
     hkl = np.array(list(itertools.product(h, h, h)))  # all evaluated Miller indices
@@ -866,25 +776,31 @@ def kinematic_scattering(tags, verbose=False):
     hkl_all = hkl.copy()
     s_g = S[reflections]
     g_hkl = g[reflections]
+    
     hkl = hkl[reflections]
     g_norm = g_norm[reflections]
 
     if verbose:
         print(f"Of the {len(g)} tested reciprocal_unit_cell points, {len(g_hkl)} "
-              f"have an excitation error less than {Sg_max:.2f} 1/nm")
+              f"have an excitation error less than {Sg_max:.2f} 1/Angstrom")
 
     # Calculate Structure Factors
     structure_factors = []
     for j in range(len(g_hkl)):
         F = 0
-        for b in range(len(tags['base'])):
-            f = feq(tags['elements'][b], np.linalg.norm(g_hkl[j]))
-            F += f * np.exp(-2 * np.pi * 1j * (hkl[j] * tags['base'][b]).sum())
+        for b in range(len(atoms)):
+            f = feq(atoms[b].symbol, np.linalg.norm(g_hkl[j]))
+            F += f * np.exp(-2 * np.pi * 1j * (hkl[j] * atoms.get_scaled_positions()[b]).sum())
 
         structure_factors.append(F)
 
     F = structure_factors = np.array(structure_factors)
+    
+    shift_x = k0 * np.tan(np.deg2rad(tags['mistilt alpha degree']))
+    shift_y = k0 * np.tan(np.deg2rad(tags['mistilt beta degree']))
 
+    g_hkl += [shift_x, shift_y, 0]
+    
     # Sort reflection in allowed and forbidden #
     allowed = np.absolute(F) > 0.000001  # allowed within numerical error
 
@@ -898,11 +814,13 @@ def kinematic_scattering(tags, verbose=False):
     F_allowed = F[allowed]
     g_norm_allowed = g_norm[allowed]
 
-    tags['allowed'] = {}
-    tags['allowed']['Sg'] = s_g_allowed
-    tags['allowed']['hkl'] = hkl_allowed
-    tags['allowed']['g'] = g_allowed
-    tags['allowed']['structure factor'] = F_allowed
+    atoms.info['diffraction'] = {}
+    dif = atoms.info['diffraction']
+    dif['allowed'] = {}
+    dif['allowed']['Sg'] = s_g_allowed
+    dif['allowed']['hkl'] = hkl_allowed
+    dif['allowed']['g'] = g_allowed
+    dif['allowed']['structure factor'] = F_allowed
 
     # Calculate Extinction Distance  Reimer 7.23
     # - makes only sense for non zero F
@@ -915,9 +833,9 @@ def kinematic_scattering(tags, verbose=False):
     thickness = tags['thickness']
     if thickness > 0.1:
         I_g = np.real(np.pi ** 2 / xi_g ** 2 * np.sin(np.pi * thickness * s_g_allowed) ** 2 / (np.pi * s_g_allowed)**2)
-        tags['allowed']['Ig'] = I_g
+        dif['allowed']['Ig'] = I_g
 
-    tags['allowed']['intensities'] = intensities = np.real(F_allowed) ** 2
+    dif['allowed']['intensities'] = intensities = np.real(F_allowed) ** 2
 
     # information of forbidden reflections
     forbidden = np.logical_not(allowed)
@@ -926,37 +844,35 @@ def kinematic_scattering(tags, verbose=False):
     g_forbidden = g_hkl[forbidden]
     F_forbidden = F[forbidden]
 
-    tags['forbidden'] = {}
-    tags['forbidden']['Sg'] = s_g_forbidden
-    tags['forbidden']['hkl'] = hkl_forbidden
-    tags['forbidden']['g'] = g_forbidden
-
-    # Dynamically Allowed Reflection
-    indices = range(len(hkl_allowed))
-    dynamic_allowed = [False] * len(hkl_forbidden)
-
-    ls = hkl_forbidden.tolist()
-
-    comb = [list(x) for x in itertools.permutations(indices, 2)]
-    for i in range(len(comb)):
-        possible = (hkl_allowed[comb[i][0]] + hkl_allowed[comb[i][1]]).tolist()
-        if possible in ls:
-            dynamic_allowed[ls.index(possible)] = True
-
-    dynamic_allowed = np.array(dynamic_allowed, dtype=int)
-    tags['dynamical allowed'] = {}
-    tags['dynamical allowed']['Sg'] = s_g_forbidden[dynamic_allowed]
-    tags['dynamical allowed']['hkl'] = hkl_forbidden[dynamic_allowed]
-    tags['dynamical allowed']['g'] = g_forbidden[dynamic_allowed]
-
-    if verbose:
-        print(f"Of the {g_forbidden.shape[0]} forbidden reflection {tags['dynamical allowed']['g'].shape[0]} "
-              f"can be dynamically activated.")
-        print(tags['dynamical allowed']['hkl'])
-
+    dif['forbidden'] = {}
+    dif['forbidden']['Sg'] = s_g_forbidden
+    dif['forbidden']['hkl'] = hkl_forbidden.copy()
+    dif['forbidden']['g'] = g_forbidden
+    
     # Make pretty labels
     hkl_label = make_pretty_labels(hkl_allowed)
-    tags['allowed']['label'] = hkl_label
+    dif['allowed']['label'] = hkl_label
+    hkl_label = make_pretty_labels(hkl_forbidden)
+    dif['forbidden']['label'] = hkl_label
+
+    # Dynamically Allowed Reflection
+    """
+    indices = range(len(hkl_allowed))
+    combinations = [list(x) for x in itertools.permutations(indices, 2)]
+    hkl_forbidden = hkl_forbidden.tolist()
+    dynamicallly_allowed = np.zeros(len(hkl_forbidden), dtype=bool)
+    for [i, j] in combinations:
+        possible = (hkl_allowed[i] + hkl_allowed[j]).tolist()
+        if possible in hkl_forbidden:
+            dynamicallly
+            _allowed[hkl_forbidden.index(possible)] = True
+    dif['forbidden']['dynamicallly_allowed'] = dynamicallly_allowed
+   
+    if verbose:
+        print(f"Of the {g_forbidden.shape[0]} forbidden reflection {dif['dynamical_allowed']['g'].shape[0]} "
+              f"can be dynamically activated.")
+        # print(dif['forbidden']['hkl'][dynamicallly_allowed])
+    """
 
     # Center of Laue Circle
     laue_circle = np.dot(tags['nearest_zone_axis'], tags['reciprocal_unit_cell'])
@@ -964,9 +880,9 @@ def kinematic_scattering(tags, verbose=False):
     laue_circle = laue_circle / np.linalg.norm(laue_circle) * k0
     laue_circle[2] = 0
 
-    tags['laue_circle'] = laue_circle
+    dif['Laue_circle'] = laue_circle
     if verbose:
-        print('laue_circle', laue_circle)
+        print('Laue_circle', laue_circle)
 
     # ###########################
     # Calculate Laue Zones (of allowed reflections)
@@ -979,17 +895,17 @@ def kinematic_scattering(tags, verbose=False):
     # Remember we have already tilted, and so the dot product is trivial and gives only the z-component.
 
     Laue_Zone = abs(np.dot(hkl_allowed, tags['nearest_zone_axis']))
-    tags['allowed']['Laue_Zone'] = Laue_Zone
+    dif['allowed']['Laue_Zone'] = Laue_Zone
 
     ZOLZ = Laue_Zone == 0
     FOLZ = Laue_Zone == 1
     SOLZ = Laue_Zone == 2
     HOLZ = Laue_Zone > 2
 
-    tags['allowed']['ZOLZ'] = ZOLZ
-    tags['allowed']['FOLZ'] = FOLZ
-    tags['allowed']['SOLZ'] = SOLZ
-    tags['allowed']['HOLZ'] = HOLZ
+    dif['allowed']['ZOLZ'] = ZOLZ
+    dif['allowed']['FOLZ'] = FOLZ
+    dif['allowed']['SOLZ'] = SOLZ
+    dif['allowed']['HOLZ'] = HOLZ
 
     if verbose:
         print(' There are {0} allowed reflections in the zero order Laue Zone'.format(ZOLZ.sum()))
@@ -1017,14 +933,15 @@ def kinematic_scattering(tags, verbose=False):
 
         zone_tags = tags['nearest_zone_axes'][str(i)]
 
-        print('Calculating Kikuchi lines for zone: ', zone_tags['hkl'])
+        if verbose:
+            print('Calculating Kikuchi lines for zone: ', zone_tags['hkl'])
 
         laue_circle = np.dot(zone_tags['hkl'], tags['reciprocal_unit_cell'])
         laue_circle = np.dot(laue_circle, rotation_matrix)
         laue_circle = laue_circle / np.linalg.norm(laue_circle) * k0
         laue_circle[2] = 0
 
-        zone_tags['laue_circle'] = laue_circle
+        zone_tags['Laue_circle'] = laue_circle
         # Rotate to nearest zone axis
 
         tags_new_zone['zone_hkl']
@@ -1056,9 +973,9 @@ def kinematic_scattering(tags, verbose=False):
         structure_factors = []
         for j in range(len(g_hkl_kikuchi2)):
             F = 0
-            for b in range(len(tags['base'])):
-                f = feq(tags['elements'][b], np.linalg.norm(g_hkl_kikuchi2[j]))
-                F += f * np.exp(-2 * np.pi * 1j * (hkl_kikuchi2[j] * tags['base'][b]).sum())
+            for b in range(len(atoms)):
+                f = feq(atoms[b].symbol, np.linalg.norm(g_hkl_kikuchi2[j]))
+                F += f * np.exp(-2 * np.pi * 1j * (hkl_kikuchi2[j] * atoms.get_scaled_positions()[b]).sum())
 
             structure_factors.append(F)
 
@@ -1077,14 +994,14 @@ def kinematic_scattering(tags, verbose=False):
         distance2 = np.sqrt(gd2[:, 0] * gd2[:, 0] + gd2[:, 1] * gd2[:, 1])
         theta2 = np.arctan(slope2)
 
-        tags['Kikuchi'] = {}
-        tags['Kikuchi']['slope'] = slope2
-        tags['Kikuchi']['distance'] = distance2
-        tags['Kikuchi']['theta'] = theta2
-        tags['Kikuchi']['hkl'] = hkl_kikuchi
-        tags['Kikuchi']['g_hkl'] = g_hkl_kikuchi
-        tags['Kikuchi']['g deficient'] = gd2
-        tags['Kikuchi']['min dist'] = gd2 + laue_circle
+        dif['Kikuchi'] = {}
+        dif['Kikuchi']['slope'] = slope2
+        dif['Kikuchi']['distance'] = distance2
+        dif['Kikuchi']['theta'] = theta2
+        dif['Kikuchi']['hkl'] = hkl_kikuchi
+        dif['Kikuchi']['g_hkl'] = g_hkl_kikuchi
+        dif['Kikuchi']['g_deficient'] = gd2
+        dif['Kikuchi']['min dist'] = gd2 + laue_circle
 
     k_g = k0
 
@@ -1119,23 +1036,23 @@ def kinematic_scattering(tags, verbose=False):
     distance = gd_length
     theta = np.arctan(slope)
 
-    tags['HOLZ'] = {}
-    tags['HOLZ']['slope'] = slope
+    dif['HOLZ'] = {}
+    dif['HOLZ']['slope'] = slope
     # a line is now given by
-    tags['HOLZ']['distance'] = distance
-    tags['HOLZ']['theta'] = theta
-    tags['HOLZ']['g deficient'] = gd
-    tags['HOLZ']['g excess'] = gd + g_allowed
-    tags['HOLZ']['g_allowed'] = g_allowed.copy()
+    dif['HOLZ']['distance'] = distance
+    dif['HOLZ']['theta'] = theta
+    dif['HOLZ']['g_deficient'] = gd
+    dif['HOLZ']['g_excess'] = gd + g_allowed
+    dif['HOLZ']['g_allowed'] = g_allowed.copy()
 
-    tags['HOLZ']['ZOLZ'] = ZOLZ
-    tags['HOLZ']['HOLZ'] = np.logical_not(ZOLZ)
-    tags['HOLZ']['FOLZ'] = FOLZ
-    tags['HOLZ']['SOLZ'] = SOLZ
-    tags['HOLZ']['HHOLZ'] = HOLZ  # even higher HOLZ
+    dif['HOLZ']['ZOLZ'] = ZOLZ
+    dif['HOLZ']['HOLZ'] = np.logical_not(ZOLZ)
+    dif['HOLZ']['FOLZ'] = FOLZ
+    dif['HOLZ']['SOLZ'] = SOLZ
+    dif['HOLZ']['HHOLZ'] = HOLZ  # even higher HOLZ
 
-    tags['HOLZ']['hkl'] = tags['allowed']['hkl']
-    tags['HOLZ']['intensities'] = intensities
+    dif['HOLZ']['hkl'] = dif['allowed']['hkl']
+    dif['HOLZ']['intensities'] = intensities
 
     print('done')
 
@@ -1183,10 +1100,10 @@ def make_pretty_labels(hkls, hex_label=False):
 
 
 def feq(element, q):
-    """Atomic form factor parametrized in 1/Angstrom but converted to 1/nm
+    """Atomic form factor parametrized in 1/Angstrom but converted to 1/Angstrom
 
     The atomic form factor is from Kirkland: Advanced Computing in Electron Microscopy 2nd edition, Appendix C.
-    From Appendix C of Kirkland, "Advanced Computing in Electron Microscopy", 2nd ed.
+    From Appendix C of Kirkland, "Advanced Computing in Electron Microscopy", 3Ard ed.
     Calculation of electron form factor for specific q:
     Using equation Kirkland C.15
 
@@ -1195,7 +1112,7 @@ def feq(element, q):
     element: string
         element name
     q: float
-        magnitude of scattering vector in 1/nm -- (=> exp(-i*g.r), physics negative convention)
+        magnitude of scattering vector in 1/Angstrom -- (=> exp(-i*g.r), physics negative convention)
 
     Returns
     -------
@@ -1213,8 +1130,7 @@ def feq(element, q):
     if not isinstance(q, (float, int)):
         raise TypeError('Magnitude of scattering vector has to be a number of type float')
 
-    q = q/10
-    # q is now magnitude of scattering vector in 1/A -- (=> exp(-i*g.r), physics negative convention)
+    # q is in magnitude of scattering vector in 1/A -- (=> exp(-i*g.r), physics negative convention)
     param = electronFF[element]
     f_lorentzian = 0
     f_gauss = 0
